%VECTOR Kalman Filter example (Object falling in air):
% Joshua Smith

x0 = 0.0; v0 = 0.0;
TMAX = 200;
g=9.8;
m=1.0; k=10.0;
dt=0.01;

clear s % Dynamics modeled by A
s.A = [[1      dt       ]; ... 
       [0 (1.0-(k/m)*dt)]];
% Measurement noise variance
MNstd = 0.4;
MNV = MNstd*MNstd;
% Process noise variance
PNstd = 0.02;
PNV = PNstd*PNstd;
% Process noise covariance matrix
s.Q = eye(2)*PNV;
% Define measurement function to return the state
s.H = eye(2);
% Define a measurement error
s.R = eye(2)*MNV; % variance

% Use control to include gravity
s.B = eye(2); % Control matrix
s.u = [0 -g*m*dt]'; % Gravitational acceleration
% Initial state:
s.x = [x0 v0]';
s.P = eye(2)*MNV;
s.detP = det(s.P); % Let's keep track of the noise by keeping detP
s.z = zeros(2,1);

% Simulate falling in air, and watch the filter track it
tru=zeros(TMAX,2); % true dynamics
tru(1,:)=[x0 v0];
detP(1,:)=s.detP;
for t=2:TMAX
   tru(t,:) = s(t-1).A * tru(t-1,:)' + s(t-1).B*s(t-1).u + PNstd*randn(2,1);
   s(t-1).z = s(t-1).H * tru(t,:)' + MNstd*randn(2,1); % create a measurement
   s(t)=kalmanf(s(t-1)); % perform a Kalman filter iteration
   detP(t)=s(t).detP; % keep track of "net" uncertainty
end

xout = zeros(TMAX,2);
zout = zeros(TMAX,2);
for t=1:TMAX
    xout(t,:) = s(t).x
    zout(t,:) = s(t).z
end


figure(1)
subplot(3,1,1);
hold on
grid on
% plot measurement of position
hzvel=plot(zout(:,1),'r.');
% plot a-posteriori state estimates:
hxvel=plot(xout(:,1),'b-');
htvel=plot(tru(:,1),'g-');
legend([hzvel hxvel htvel],'observations','Kalman output','true dynamics',0)
title(['Position of object falling in air, Meas Nz Var= ' num2str(MNV) ' Proc Nz Var= ' num2str(PNV)])
hold off

subplot(3,1,2)
hold on
grid on
% plot measurement of velocity
hzvel=plot(zout(:,2),'r.');
% plot a-posteriori state estimates:
hxvel=plot(xout(:,2),'b-');
htvel=plot(tru(:,2),'g-');
legend([hzvel hxvel htvel],'observations','Kalman output','true dynamics',0)
title('Velocity of object falling in air ')
hold off

subplot(3,1,3)
grid on
% plot detP
hdetP=plot(detP,'r.');
%legend([hzvel hxvel htvel],'observations','Kalman output','true dynamics',0)
title('Det(P) ')
hold off
